'use strict';

(function($) {
  $(document).ready(function() {
    if (!$('.woofs-wrap').length) {
      return;
    }

    $('.woofs-wrap').each(function() {
      woofs_init($(this));
    });
  });

  $(document).on('found_variation', function(e, t) {
    var $wrap = $(e['target']).closest('.woofs-wrap');
    var $product = $(e['target']).closest('.woofs-product');
    var $products = $(e['target']).closest('.woofs-products');

    if ($product.length) {
      var _price = $product.attr('data-price');

      if (isNaN(_price)) {
        _price = t['display_price'] *
            parseInt($product.attr('data-price')) / 100;
      }

      $product.find('.woofs-price-ori').hide();
      $product.find('.woofs-price-new').
          html(woofs_format_price(_price)).
          show();

      if (t['is_purchasable'] && t['is_in_stock']) {
        $product.attr('data-id', t['variation_id']);
        $product.attr('data-price-ori', t['display_price']);

        // change attributes
        var attrs = {};

        $product.find('select[name^="attribute_"]').each(function() {
          var attr_name = $(this).attr('name');

          attrs[attr_name] = $(this).val();
        });

        $product.attr('data-attrs', JSON.stringify(attrs));
      } else {
        $product.attr('data-id', 0);
        $product.attr('data-attrs', '');
      }

      // change availability
      if (t['availability_html'] && t['availability_html'] !== '') {
        $product.find('.woofs-availability').
            html(t['availability_html']).
            show();
      } else {
        $product.find('.woofs-availability').html('').hide();
      }

      if (t['image']['url'] && t['image']['srcset']) {
        // change image
        $product.find('.woofs-thumb-ori').hide();
        $product.find('.woofs-thumb-new').
            html('<img src="' + t['image']['url'] + '" srcset="' +
                t['image']['srcset'] + '"/>').
            show();
      }

      if (woofs_vars.change_image === 'no') {
        // prevent changing the main image
        $(e['target']).
            closest('.variations_form').
            trigger('reset_image');
      }

      // prevent changing SKU
      $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.product_meta .sku').
          html($products.attr('data-product-sku'));
    } else {
      $wrap = $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.woofs-wrap');
      $products = $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.woofs-products');

      if ($products.length &&
          !$(e['target']).closest('.woosb-product').length &&
          !$(e['target']).closest('.woobt-product').length &&
          !$(e['target']).closest('.woosg-product').length) {
        $products.attr('data-product-price', t['display_price']);
        $products.attr('data-product-sku', t['sku']);
      }
    }

    if ($wrap.length) {
      woofs_init($wrap);
    }
  });

  $(document).on('reset_data', function(e) {
    var $wrap = $(e['target']).closest('.woofs-wrap');
    var $product = $(e['target']).closest('.woofs-product');
    var $products = $(e['target']).closest('.woofs-products');

    if ($product.length) {
      $product.attr('data-id', 0);
      $product.attr('data-attrs', '');

      // reset availability
      $product.find('.woofs-availability').html('').hide();

      // reset thumb
      $product.find('.woofs-thumb-new').hide();
      $product.find('.woofs-thumb-ori').show();

      // reset SKU
      $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.product_meta .sku').
          html($products.attr('data-product-sku'));
    } else {
      $wrap = $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.woofs-wrap');
      $products = $(e['target']).
          closest(woofs_vars.summary_selector).
          find('.woofs-products');

      if ($products.length &&
          !$(e['target']).closest('.woosb-product').length &&
          !$(e['target']).closest('.woobt-product').length &&
          !$(e['target']).closest('.woosg-product').length) {
        $products.attr('data-product-price', 0);
        $products.attr('data-product-sku', '');
      }
    }

    if ($wrap.length) {
      woofs_init($wrap);
    }
  });

  $(document).on('woovr_selected', function(e, selected, variations) {
    var $wrap = variations.closest('.woofs-wrap');
    var $product = variations.closest('.woofs-product');
    var $products = variations.closest('.woofs-products');
    var purchasable = selected.attr('data-purchasable');
    var price = selected.attr('data-price');
    var id = selected.attr('data-id');
    var sku = selected.attr('data-sku');
    var image_src = selected.attr('data-imagesrc');

    if ($product.length) {
      if (purchasable === 'yes') {
        $product.attr('data-id', id);
        $product.attr('data-price-ori', price);

        // change image
        if (image_src !== undefined && image_src !== '') {
          $product.find('.woofs-thumb-ori').hide();
          $product.find('.woofs-thumb-new').
              html('<img src="' + image_src + '"/>').
              show();
        }

        // change price
        var new_price = $product.attr('data-price');

        if (isNaN(new_price)) {
          new_price = price * parseFloat(new_price) / 100;
        }

        $product.find('.woofs-price-ori').hide();
        $product.find('.woofs-price-new').
            html(woofs_price_html(price, new_price)).
            show();

        // change attributes
        var attrs = {};

        $product.find('select[name^="attribute_"]').each(function() {
          var attr_name = $(this).attr('name');

          attrs[attr_name] = $(this).val();
        });

        $product.attr('data-attrs', JSON.stringify(attrs));
      } else {
        // reset data
        $product.attr('data-id', 0);
        $product.attr('data-attrs', '');
        $product.attr('data-price-ori', 0);

        // reset image
        $product.find('.woofs-thumb-ori').show();
        $product.find('.woofs-thumb-new').html('').hide();

        // reset price
        $product.find('.woofs-price-ori').show();
        $product.find('.woofs-price-new').html('').hide();
      }

      // prevent changing SKU
      variations.closest(woofs_vars.summary_selector).
          find('.product_meta .sku').
          html($products.attr('data-product-sku'));
    } else {
      $wrap = variations.closest(woofs_vars.summary_selector).
          find('.woofs-wrap');
      $products = variations.closest(woofs_vars.summary_selector).
          find('.woofs-products');

      if ($products.length &&
          !variations.closest('.woosb-product').length &&
          !variations.closest('.woobt-product').length &&
          !variations.closest('.woosg-product').length) {
        if (purchasable === 'yes') {
          $products.attr('data-product-price', price);
        } else {
          $products.attr('data-product-price', 0);
        }
      }
    }

    if ($wrap.length) {
      woofs_init($wrap);
    }
  });

  $(document).on('click touch', '.single_add_to_cart_button', function(e) {
    if ($(this).hasClass('woofs-disabled')) {
      e.preventDefault();
    }
  });
})(jQuery);

function woofs_init($wrap) {
  var container = woofs_container($wrap);
  var $container = $wrap.closest(container);
  var $products = $wrap.find('.woofs-products');
  var $ids = $container.find('.woofs-ids');

  if ((woofs_vars.position === 'before') &&
      ($products.attr('data-product-type') === 'variable') &&
      (($products.attr('data-variables') === 'no') ||
          (woofs_vars.variations_selector === 'wpc_radio'))) {
    $wrap.insertAfter($ids);
  }

  woofs_check_ready($container);
  woofs_calc_price($container);
  woofs_save_ids($container);
}

function woofs_check_ready($container) {
  var is_selection = false;
  var selection_name = '';
  var $alert = $container.find('.woofs-alert');
  var $btn = $container.find('.single_add_to_cart_button');

  $container.find('.woofs-product').each(function() {
    var $this = jQuery(this);

    if ($this.attr('data-qty') == 0) {
      $this.addClass('woofs-hide');
    } else {
      $this.removeClass('woofs-hide');
    }

    if ((
        $this.attr('data-qty') > 0
    ) && (
        $this.attr('data-id') == 0
    )) {
      is_selection = true;

      if (selection_name === '') {
        selection_name = $this.attr('data-name');
      }
    }
  });

  if (is_selection) {
    $btn.addClass('woofs-disabled');
    $alert.html(woofs_vars.alert_selection.replace('[name]',
        '<strong>' + selection_name + '</strong>')).slideDown();

    jQuery(document).trigger('woofs_check_ready', [false, is_selection]);
  } else {
    $alert.html('').slideUp();
    $btn.removeClass('woofs-disabled');

    // ready
    jQuery(document).trigger('woofs_check_ready', [true, is_selection]);
  }
}

function woofs_calc_price($container) {
  var $products = $container.find('.woofs-products');
  var $total = $container.find('.woofs-total');
  var total = 0, total_ori = 0;
  var total_html = '', total_ori_html = '';
  var ori_price = parseFloat($products.attr('data-product-price'));
  var ori_qty = parseInt($container.find('input.qty').val());
  var price_selector = woofs_vars.summary_selector + ' > .price';

  if ((woofs_vars.change_price === 'yes_custom') &&
      (woofs_vars.price_selector != null) &&
      (woofs_vars.price_selector !== '')) {
    price_selector = woofs_vars.price_selector;
  }

  $container.find('.woofs-product').each(function() {
    var $this = jQuery(this);
    var item_total = 0;

    if ($this.attr('data-qty') > 0 && $this.attr('data-id') > 0) {
      if (isNaN($this.attr('data-price'))) {
        //is percent
        item_total = (
            $this.attr('data-price-ori') *
            parseInt($this.attr('data-price')) / 100
        ) * $this.attr('data-qty');
      } else {
        item_total = $this.attr('data-price') * $this.attr('data-qty');
      }

      if (woofs_vars.show_price === 'total') {
        $this.find('.woofs-price-ori').hide();
        $this.find('.woofs-price-new').
            html(woofs_format_price(item_total)).
            show();
      }

      //calc total
      total += item_total;
    } else {
      $this.find('.woofs-price-new').hide();
      $this.find('.woofs-price-ori').show();
    }
  });

  if (total > 0) {
    total_html = woofs_format_price(total);
    $total.html(woofs_vars.total_price_text + ' ' + total_html).
        slideDown();

    total_ori = ori_price * ori_qty + total;
    total_ori_html = woofs_format_price(total_ori);

    // change the main price
    if (woofs_vars.change_price !== 'no') {
      jQuery(price_selector).html(total_ori_html);
    }
  } else {
    $total.html('').slideUp();

    // change the main price
    if (woofs_vars.change_price !== 'no') {
      jQuery(price_selector).html(woofs_format_price(ori_price));
    }
  }

  jQuery(document).trigger('woofs_calc_price', [total, total_html]);
}

function woofs_save_ids($container) {
  var $ids = $container.find('.woofs-ids');
  var items = new Array();

  $container.find('.woofs-product').each(function() {
    var $this = jQuery(this);
    var id = parseInt($this.attr('data-id'));
    var price = jQuery(this).attr('data-price');
    var qty = parseFloat($this.attr('data-qty'));
    var attrs = $this.attr('data-attrs');

    if ((id > 0) && (qty > 0)) {
      if (attrs != undefined) {
        attrs = encodeURIComponent(attrs);
      } else {
        attrs = '';
      }

      items.push(id + '/' + price + '/' + qty + '/' + attrs);
    }
  });

  if (items.length > 0) {
    $ids.val(items.join(','));
  } else {
    $ids.val('');
  }

  jQuery(document).trigger('woofs_save_ids', [items]);
}

function woofs_format_money(number, places, symbol, thousand, decimal) {
  number = number || 0;
  places = !isNaN(places = Math.abs(places)) ? places : 2;
  symbol = symbol !== undefined ? symbol : '$';
  thousand = thousand || ',';
  decimal = decimal || '.';
  var negative = number < 0 ? '-' : '',
      i = parseInt(number = Math.abs(+number || 0).toFixed(places), 10) +
          '',
      j = 0;
  if (i.length > 3) {
    j = i.length % 3;
  }
  return symbol + negative + (
      j ? i.substr(0, j) + thousand : ''
  ) + i.substr(j).replace(/(\d{3})(?=\d)/g, '$1' + thousand) + (
      places ? decimal + Math.abs(number - i).toFixed(places).slice(2) : ''
  );
}

function woofs_format_price(total, prefix) {
  var total_html = '<span class="woocommerce-Price-amount amount">';
  var total_formatted = woofs_format_money(total, woofs_vars.price_decimals,
      '', woofs_vars.price_thousand_separator,
      woofs_vars.price_decimal_separator);

  switch (woofs_vars.price_format) {
    case '%1$s%2$s':
      //left
      total_html += '<span class="woocommerce-Price-currencySymbol">' +
          woofs_vars.currency_symbol + '</span>' + total_formatted;
      break;
    case '%1$s %2$s':
      //left with space
      total_html += '<span class="woocommerce-Price-currencySymbol">' +
          woofs_vars.currency_symbol + '</span> ' + total_formatted;
      break;
    case '%2$s%1$s':
      //right
      total_html += total_formatted +
          '<span class="woocommerce-Price-currencySymbol">' +
          woofs_vars.currency_symbol + '</span>';
      break;
    case '%2$s %1$s':
      //right with space
      total_html += total_formatted +
          ' <span class="woocommerce-Price-currencySymbol">' +
          woofs_vars.currency_symbol + '</span>';
      break;
    default:
      //default
      total_html += '<span class="woocommerce-Price-currencySymbol">' +
          woofs_vars.currency_symbol + '</span> ' + total_formatted;
  }

  total_html += '</span>';

  if (prefix != null) {
    return prefix + total_html;
  }

  return total_html;
}

function woofs_price_html(regular_price, sale_price) {
  var price_html = '';

  if (sale_price < regular_price) {
    price_html = '<del>' + woofs_format_price(regular_price) +
        '</del> <ins>' +
        woofs_format_price(sale_price) + '</ins>';
  } else {
    price_html = woofs_format_price(regular_price);
  }

  return price_html;
}

function woofs_container($wrap) {
  if ($wrap.closest('form.cart').length) {
    return 'form.cart';
  }

  if ($wrap.closest('.summary').length) {
    return '.summary';
  }

  return 'body.single-product';
}