<?php
/*
Plugin Name: WPC Force Sells for WooCommerce
Plugin URI: https://wpclever.net/
Description: Create a deal that combines various related products and put them for sale altogether.
Version: 4.4.0
Author: WPClever
Author URI: https://wpclever.net
Text Domain: wpc-force-sells
Domain Path: /languages/
Requires at least: 4.0
Tested up to: 5.8.1
WC requires at least: 3.0
WC tested up to: 5.7.1
*/

defined( 'ABSPATH' ) || exit;

! defined( 'WOOFS_VERSION' ) && define( 'WOOFS_VERSION', '4.4.0' );
! defined( 'WOOFS_URI' ) && define( 'WOOFS_URI', plugin_dir_url( __FILE__ ) );
! defined( 'WOOFS_SUPPORT' ) && define( 'WOOFS_SUPPORT', 'https://wpclever.net/support?utm_source=support&utm_medium=woofs&utm_campaign=wporg' );
! defined( 'WOOFS_REVIEWS' ) && define( 'WOOFS_REVIEWS', 'https://wordpress.org/support/plugin/wpc-force-sells/reviews/?filter=5' );
! defined( 'WOOFS_CHANGELOG' ) && define( 'WOOFS_CHANGELOG', 'https://wordpress.org/plugins/wpc-force-sells/#developers' );
! defined( 'WOOFS_DISCUSSION' ) && define( 'WOOFS_DISCUSSION', 'https://wordpress.org/support/plugin/wpc-force-sells' );
! defined( 'WPC_URI' ) && define( 'WPC_URI', WOOFS_URI );

include 'includes/wpc-dashboard.php';
include 'includes/wpc-menu.php';
include 'includes/wpc-kit.php';
include 'includes/wpc-notice.php';

if ( ! function_exists( 'woofs_init' ) ) {
	add_action( 'plugins_loaded', 'woofs_init', 11 );

	function woofs_init() {
		// Load textdomain
		load_plugin_textdomain( 'wpc-force-sells', false, basename( __DIR__ ) . '/languages/' );

		if ( ! function_exists( 'WC' ) || ! version_compare( WC()->version, '3.0', '>=' ) ) {
			add_action( 'admin_notices', 'woofs_notice_wc' );

			return;
		}

		if ( ! class_exists( 'WPClever_Woofs' ) && class_exists( 'WC_Product' ) ) {
			class WPClever_Woofs {
				protected static $localization = array();
				protected static $woofs_types = array( 'simple' );

				function __construct() {
					// Init
					add_action( 'init', array( $this, 'woofs_init' ) );

					// Menu
					add_action( 'admin_menu', array( $this, 'woofs_admin_menu' ) );

					// Enqueue frontend scripts
					add_action( 'wp_enqueue_scripts', array( $this, 'woofs_wp_enqueue_scripts' ) );

					// Enqueue backend scripts
					add_action( 'admin_enqueue_scripts', array( $this, 'woofs_admin_enqueue_scripts' ) );

					// Backend AJAX search
					add_action( 'wp_ajax_woofs_get_search_results', array( $this, 'woofs_get_search_results' ) );

					// Product data tabs
					add_filter( 'woocommerce_product_data_tabs', array( $this, 'woofs_product_data_tabs' ), 10, 1 );

					// Product data panels
					add_action( 'woocommerce_product_data_panels', array( $this, 'woofs_product_data_panels' ) );
					add_action( 'woocommerce_process_product_meta', array( $this, 'woofs_save_option_field' ) );

					// Add to cart form & button
					if ( get_option( '_woofs_position', 'before' ) === 'before' ) {
						add_action( 'woocommerce_before_add_to_cart_form', array( $this, 'woofs_add_to_cart_form' ) );
					} else {
						add_action( 'woocommerce_after_add_to_cart_form', array( $this, 'woofs_add_to_cart_form' ) );
					}
					add_action( 'woocommerce_before_add_to_cart_button', array( $this, 'woofs_add_to_cart_button' ) );

					// Add to cart
					add_filter( 'woocommerce_add_to_cart_validation', array(
						$this,
						'woofs_add_to_cart_validation'
					), 10, 2 );
					add_action( 'woocommerce_add_to_cart', array( $this, 'woofs_add_to_cart' ), 10, 6 );
					add_filter( 'woocommerce_add_cart_item_data', array( $this, 'woofs_add_cart_item_data' ), 10, 2 );
					add_filter( 'woocommerce_get_cart_item_from_session', array(
						$this,
						'woofs_get_cart_item_from_session'
					), 10, 2 );

					// Cart item
					add_filter( 'woocommerce_cart_item_remove_link', array(
						$this,
						'woofs_cart_item_remove_link'
					), 10, 2 );
					add_filter( 'woocommerce_cart_item_name', array( $this, 'woofs_item_name' ), 10, 2 );
					add_filter( 'woocommerce_cart_item_quantity', array(
						$this,
						'woofs_cart_item_quantity'
					), 10, 3 );
					add_action( 'woocommerce_cart_item_removed', array( $this, 'woofs_cart_item_removed' ), 10, 2 );

					// Cart contents count
					add_filter( 'woocommerce_cart_contents_count', array( $this, 'woofs_cart_contents_count' ) );
					add_filter( 'woocommerce_get_item_count', array( $this, 'woofs_get_item_count' ), 10, 3 );

					// Order item
					add_action( 'woocommerce_checkout_create_order_line_item', array(
						$this,
						'woofs_add_order_item_meta'
					), 10, 3 );
					add_filter( 'woocommerce_order_item_name', array( $this, 'woofs_item_name' ), 10, 2 );

					add_action( 'woocommerce_new_order_item', array(
						$this,
						'woofs_new_order_item'
					), 10, 3 );
					add_filter( 'woocommerce_hidden_order_itemmeta', array(
						$this,
						'woofs_hidden_order_item_meta'
					), 10, 1 );
					add_action( 'woocommerce_before_order_itemmeta', array(
						$this,
						'woofs_before_order_item_meta'
					), 10, 1 );

					if ( get_option( '_woofs_hide_linked', 'no' ) === 'yes' ) {
						// Hide on cart & checkout page
						add_filter( 'woocommerce_cart_item_visible', array( $this, 'woofs_item_visible' ), 10, 2 );
						add_filter( 'woocommerce_order_item_visible', array( $this, 'woofs_item_visible' ), 10, 2 );
						add_filter( 'woocommerce_checkout_cart_item_visible', array(
							$this,
							'woofs_item_visible'
						), 10, 2 );

						// Hide on mini-cart
						add_filter( 'woocommerce_widget_cart_item_visible', array(
							$this,
							'woofs_item_visible'
						), 10, 2 );
					}

					// Admin
					add_filter( 'display_post_states', array( $this, 'woofs_display_post_states' ), 10, 2 );

					// Add settings link
					add_filter( 'plugin_action_links', array( $this, 'woofs_action_links' ), 10, 2 );
					add_filter( 'plugin_row_meta', array( $this, 'woofs_row_meta' ), 10, 2 );

					// Get cart contents
					add_filter( 'woocommerce_get_cart_contents', array( $this, 'woofs_get_cart_contents' ), 10, 1 );

					// Search filters
					if ( get_option( '_woofs_search_sku', 'no' ) === 'yes' ) {
						add_filter( 'pre_get_posts', array( $this, 'woofs_search_sku' ), 99 );
					}
					if ( get_option( '_woofs_search_exact', 'no' ) === 'yes' ) {
						add_action( 'pre_get_posts', array( $this, 'woofs_search_exact' ), 99 );
					}
					if ( get_option( '_woofs_search_sentence', 'no' ) === 'yes' ) {
						add_action( 'pre_get_posts', array( $this, 'woofs_search_sentence' ), 99 );
					}
				}

				function woofs_init() {
					self::$localization = (array) get_option( 'woofs_localization' );
				}

				function woofs_localization( $key = '', $default = '' ) {
					$str = '';

					if ( ! empty( $key ) && ! empty( self::$localization[ $key ] ) ) {
						$str = self::$localization[ $key ];
					} elseif ( ! empty( $default ) ) {
						$str = $default;
					}

					return apply_filters( 'woofs_localization_' . $key, $str );
				}

				function woofs_admin_menu() {
					add_submenu_page( 'wpclever', esc_html__( 'WPC Force Sells', 'wpc-force-sells' ), esc_html__( 'Force Sells', 'wpc-force-sells' ), 'manage_options', 'wpclever-woofs', array(
						&$this,
						'woofs_admin_menu_content'
					) );
				}

				function woofs_admin_menu_content() {
					add_thickbox();
					$active_tab = isset( $_GET['tab'] ) ? esc_attr( $_GET['tab'] ) : 'settings';
					?>
                    <div class="wpclever_settings_page wrap">
                        <h1 class="wpclever_settings_page_title"><?php echo esc_html__( 'WPC Force Sells', 'wpc-force-sells' ) . ' ' . WOOFS_VERSION; ?></h1>
                        <div class="wpclever_settings_page_desc about-text">
                            <p>
								<?php printf( esc_html__( 'Thank you for using our plugin! If you are satisfied, please reward it a full five-star %s rating.', 'wpc-force-sells' ), '<span style="color:#ffb900">&#9733;&#9733;&#9733;&#9733;&#9733;</span>' ); ?>
                                <br/>
                                <a href="<?php echo esc_url( WOOFS_REVIEWS ); ?>"
                                   target="_blank"><?php esc_html_e( 'Reviews', 'wpc-force-sells' ); ?></a> | <a
                                        href="<?php echo esc_url( WOOFS_CHANGELOG ); ?>"
                                        target="_blank"><?php esc_html_e( 'Changelog', 'wpc-force-sells' ); ?></a>
                                | <a href="<?php echo esc_url( WOOFS_DISCUSSION ); ?>"
                                     target="_blank"><?php esc_html_e( 'Discussion', 'wpc-force-sells' ); ?></a>
                            </p>
                        </div>
                        <div class="wpclever_settings_page_nav">
                            <h2 class="nav-tab-wrapper">
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-woofs&tab=how' ); ?>"
                                   class="<?php echo $active_tab === 'how' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'How to use?', 'wpc-force-sells' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-woofs&tab=settings' ); ?>"
                                   class="<?php echo $active_tab === 'settings' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Settings', 'wpc-force-sells' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-woofs&tab=localization' ); ?>"
                                   class="<?php echo $active_tab === 'localization' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>">
									<?php esc_html_e( 'Localization', 'wpc-force-sells' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-woofs&tab=premium' ); ?>"
                                   class="<?php echo $active_tab === 'premium' ? 'nav-tab nav-tab-active' : 'nav-tab'; ?>"
                                   style="color: #c9356e;">
									<?php esc_html_e( 'Premium Version', 'wpc-force-sells' ); ?>
                                </a>
                                <a href="<?php echo esc_url( admin_url( 'admin.php?page=wpclever-kit' ) ); ?>"
                                   class="nav-tab">
									<?php esc_html_e( 'Essential Kit', 'wpc-force-sells' ); ?>
                                </a>
                            </h2>
                        </div>
                        <div class="wpclever_settings_page_content">
							<?php if ( $active_tab === 'how' ) { ?>
                                <div class="wpclever_settings_page_content_text">
                                    <p>
										<?php esc_html_e( 'When adding/editing the product you can choose Force Sells tab then add some products with the new price and quantity.', 'wpc-force-sells' ); ?>
                                    </p>
                                </div>
							<?php } elseif ( $active_tab === 'settings' ) { ?>
                                <form method="post" action="options.php">
									<?php wp_nonce_field( 'update-options' ) ?>
                                    <table class="form-table">
                                        <tr class="heading">
                                            <th colspan="2">
												<?php esc_html_e( 'General', 'wpc-force-sells' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Position', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_position">
                                                    <option
                                                            value="before" <?php echo esc_attr( get_option( '_woofs_position', 'before' ) === 'before' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Above add to cart button', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="after" <?php echo esc_attr( get_option( '_woofs_position', 'before' ) === 'after' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Under add to cart button', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Choose the position to show the products list.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Show thumbnail', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_show_thumb">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_show_thumb', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_show_thumb', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Show price', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_show_price">
                                                    <option value="yes" <?php echo esc_attr( get_option( '_woofs_show_price', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Price', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option value="total" <?php echo esc_attr( get_option( '_woofs_show_price', 'yes' ) === 'total' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Total', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_show_price', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Show short description', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_show_description">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_show_description', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_show_description', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Link to individual product', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_link">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_link', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the same tab', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="yes_blank" <?php echo esc_attr( get_option( '_woofs_link', 'yes' ) === 'yes_blank' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the new tab', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="yes_popup" <?php echo esc_attr( get_option( '_woofs_link', 'yes' ) === 'yes_popup' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open quick view popup', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_link', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select> <span class="description">If you choose "Open quick view popup", please install <a
                                                            href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=woo-smart-quick-view&TB_iframe=true&width=800&height=550' ) ); ?>"
                                                            class="thickbox" title="Install WPC Smart Quick View">WPC Smart Quick View</a> to make it work.</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Variations selector', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_variations_selector">
                                                    <option
                                                            value="default" <?php echo esc_attr( get_option( '_woofs_variations_selector', 'default' ) === 'default' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Default', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="wpc_radio" <?php echo esc_attr( get_option( '_woofs_variations_selector', 'default' ) === 'wpc_radio' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Use WPC Variations Radio Buttons', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select> <span class="description">If you choose "Use WPC Variations Radio Buttons", please install <a
                                                            href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=wpc-variations-radio-buttons&TB_iframe=true&width=800&height=550' ) ); ?>"
                                                            class="thickbox"
                                                            title="Install WPC Variations Radio Buttons">WPC Variations Radio Buttons</a> to make it work.</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Change image', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_change_image">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_change_image', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_change_image', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Change the main product image when choosing the variation of variable products.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Change price', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_change_price">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_change_price', 'yes' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="yes_custom" <?php echo esc_attr( get_option( '_woofs_change_price', 'yes' ) === 'yes_custom' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, custom selector', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_change_price', 'yes' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select> <input type="text" name="_woofs_change_price_custom"
                                                                 value="<?php echo get_option( '_woofs_change_price_custom', '.summary > .price' ); ?>"
                                                                 placeholder=".summary > .price"/>
                                                <span class="description"><?php esc_html_e( 'Change the main product price when choosing the variation of variable products. It uses JavaScript to change product price so it is very dependent on theme’s HTML. If it cannot find and update the product price, please contact us and we can help you adjust the JS file.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr class="heading">
                                            <th colspan="2">
												<?php esc_html_e( 'Cart & Checkout', 'wpc-force-sells' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Hide linked products', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_hide_linked">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_hide_linked', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, show the main product only', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_hide_linked', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Cart content count', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_cart_contents_count">
                                                    <option
                                                            value="main" <?php echo esc_attr( get_option( '_woofs_cart_contents_count', 'both' ) === 'main' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Main products only', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="linked" <?php echo esc_attr( get_option( '_woofs_cart_contents_count', 'both' ) === 'linked' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Linked products only', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="both" <?php echo esc_attr( get_option( '_woofs_cart_contents_count', 'both' ) === 'both' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Both main and linked products', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr class="heading" id="search">
                                            <th colspan="2">
												<?php esc_html_e( 'Search', 'wpc-force-sells' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Search limit', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input name="_woofs_search_limit" type="number" min="1"
                                                       max="500"
                                                       value="<?php echo get_option( '_woofs_search_limit', 10 ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Search by SKU', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_search_sku">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_search_sku', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_search_sku', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Search by ID', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_search_id">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_search_id', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_search_id', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Search by ID when only entered the numeric.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Search exact', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_search_exact">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_search_exact', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_search_exact', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Match whole product title or content?', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Search sentence', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_search_sentence">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_search_sentence', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_search_sentence', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'Do a phrase search?', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Accept same products', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <select name="_woofs_search_same">
                                                    <option
                                                            value="yes" <?php echo esc_attr( get_option( '_woofs_search_same', 'no' ) === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes', 'wpc-force-sells' ); ?>
                                                    </option>
                                                    <option
                                                            value="no" <?php echo esc_attr( get_option( '_woofs_search_same', 'no' ) === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-force-sells' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description"><?php esc_html_e( 'If yes, a product can be added many times.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
                                                <input type="submit" name="submit" class="button button-primary"
                                                       value="<?php esc_html_e( 'Update Options', 'wpc-force-sells' ); ?>"/>
                                                <input type="hidden" name="action" value="update"/>
                                                <input type="hidden" name="page_options"
                                                       value="_woofs_position,_woofs_show_thumb,_woofs_show_price,_woofs_show_description,_woofs_variations_selector,_woofs_link,_woofs_change_image,_woofs_change_price,_woofs_change_price_custom,_woofs_hide_linked,_woofs_cart_contents_count,_woofs_search_limit,_woofs_search_sku,_woofs_search_id,_woofs_search_exact,_woofs_search_sentence,_woofs_search_same"/>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } elseif ( $active_tab === 'localization' ) { ?>
                                <form method="post" action="options.php">
									<?php wp_nonce_field( 'update-options' ) ?>
                                    <table class="form-table">
                                        <tr class="heading">
                                            <th scope="row"><?php esc_html_e( 'General', 'wpc-force-sells' ); ?></th>
                                            <td>
												<?php esc_html_e( 'Leave blank to use the default text and its equivalent translation in multiple languages.', 'wpc-force-sells' ); ?>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Additional price', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input type="text" name="woofs_localization[additional]"
                                                       class="regular-text"
                                                       value="<?php echo esc_attr( self::woofs_localization( 'additional' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( 'Additional price:', 'wpc-force-sells' ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Linked', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input type="text" name="woofs_localization[linked]"
                                                       class="regular-text"
                                                       value="<?php echo esc_attr( self::woofs_localization( 'linked' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( '(linked to %s)', 'wpc-force-sells' ); ?>"/>
                                                <span class="description"><?php esc_html_e( 'The text behind force sells products. Use "%s" for the main product name.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Default above text', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input type="text" name="woofs_localization[above_text]"
                                                       class="large-text"
                                                       value="<?php echo esc_attr( self::woofs_localization( 'above_text' ) ); ?>"/>
                                                <span class="description"><?php esc_html_e( 'The default text above products list. You can overwrite it in product settings.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Default under text', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input type="text" name="woofs_localization[under_text]"
                                                       class="large-text"
                                                       value="<?php echo esc_attr( self::woofs_localization( 'under_text' ) ); ?>"/>
                                                <span class="description"><?php esc_html_e( 'The default text under products list. You can overwrite it in product settings.', 'wpc-force-sells' ); ?></span>
                                            </td>
                                        </tr>
                                        <tr class="heading">
                                            <th colspan="2">
												<?php esc_html_e( 'Alert', 'wpc-force-sells' ); ?>
                                            </th>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Require selection', 'wpc-force-sells' ); ?></th>
                                            <td>
                                                <input type="text" name="woofs_localization[alert_selection]"
                                                       class="large-text"
                                                       value="<?php echo esc_attr( self::woofs_localization( 'alert_selection' ) ); ?>"
                                                       placeholder="<?php esc_attr_e( 'Please select a purchasable variation for [name] before adding this product to the cart.', 'wpc-force-sells' ); ?>"/>
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
                                                <input type="submit" name="submit" class="button button-primary"
                                                       value="<?php esc_attr_e( 'Update Options', 'wpc-force-sells' ); ?>"/>
                                                <input type="hidden" name="action" value="update"/>
                                                <input type="hidden" name="page_options" value="woofs_localization"/>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } elseif ( $active_tab === 'premium' ) { ?>
                                <div class="wpclever_settings_page_content_text">
                                    <p>
                                        Get the Premium Version just $29! <a
                                                href="https://wpclever.net/downloads/wpc-force-sells?utm_source=pro&utm_medium=woofs&utm_campaign=wporg"
                                                target="_blank">https://wpclever.net/downloads/wpc-force-sells</a>
                                    </p>
                                    <p><strong>Extra features for Premium Version:</strong></p>
                                    <ul style="margin-bottom: 0">
                                        <li>- Add a variable product or a specific variation of a product.</li>
                                        <li>- Get the lifetime update & premium support.</li>
                                    </ul>
                                </div>
							<?php } ?>
                        </div>
                    </div>
					<?php
				}

				function woofs_wp_enqueue_scripts() {
					wp_enqueue_style( 'woofs-frontend', WOOFS_URI . 'assets/css/frontend.css' );
					wp_enqueue_script( 'woofs-frontend', WOOFS_URI . 'assets/js/frontend.js', array( 'jquery' ), WOOFS_VERSION, true );

					wp_localize_script( 'woofs-frontend', 'woofs_vars', array(
							'price_decimals'           => wc_get_price_decimals(),
							'price_format'             => get_woocommerce_price_format(),
							'price_thousand_separator' => wc_get_price_thousand_separator(),
							'price_decimal_separator'  => wc_get_price_decimal_separator(),
							'currency_symbol'          => get_woocommerce_currency_symbol(),
							'position'                 => get_option( '_woofs_position', 'before' ),
							'show_price'               => get_option( '_woofs_show_price', 'yes' ),
							'change_image'             => get_option( '_woofs_change_image', 'yes' ),
							'change_price'             => get_option( '_woofs_change_price', 'yes' ),
							'price_selector'           => get_option( '_woofs_change_price_custom', '' ),
							'summary_selector'         => apply_filters( 'woofs_summary_selector', '.summary' ),
							'variations_selector'      => get_option( '_woofs_variations_selector', 'default' ),
							'total_price_text'         => self::woofs_localization( 'additional', esc_html__( 'Additional price:', 'wpc-force-sells' ) ),
							'alert_selection'          => self::woofs_localization( 'alert_selection', esc_html__( 'Please select a purchasable variation for [name] before adding this product to the cart.', 'wpc-force-sells' ) ),
						)
					);
				}

				function woofs_admin_enqueue_scripts() {
					wp_enqueue_style( 'hint', WOOFS_URI . 'assets/css/hint.css' );
					wp_enqueue_style( 'woofs-backend', WOOFS_URI . 'assets/css/backend.css' );
					wp_enqueue_script( 'woofs-backend', WOOFS_URI . 'assets/js/backend.js', array(
						'jquery',
						'jquery-ui-sortable'
					), WOOFS_VERSION, true );
				}

				function woofs_action_links( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$settings         = '<a href="' . admin_url( 'admin.php?page=wpclever-woofs&tab=settings' ) . '">' . esc_html__( 'Settings', 'wpc-force-sells' ) . '</a>';
						$links['premium'] = '<a href="' . admin_url( 'admin.php?page=wpclever-woofs&tab=premium' ) . '">' . esc_html__( 'Premium Version', 'wpc-force-sells' ) . '</a>';
						array_unshift( $links, $settings );
					}

					return (array) $links;
				}

				function woofs_row_meta( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$row_meta = array(
							'support' => '<a href="' . esc_url( WOOFS_SUPPORT ) . '" target="_blank">' . esc_html__( 'Support', 'wpc-force-sells' ) . '</a>',
						);

						return array_merge( $links, $row_meta );
					}

					return (array) $links;
				}

				function woofs_cart_item_removed( $cart_item_key, $cart ) {
					if ( isset( $cart->removed_cart_contents[ $cart_item_key ]['woofs_separately'], $cart->removed_cart_contents[ $cart_item_key ]['woofs_keys'] ) && ( $cart->removed_cart_contents[ $cart_item_key ]['woofs_separately'] !== 'on' ) ) {
						$woofs_keys = $cart->removed_cart_contents[ $cart_item_key ]['woofs_keys'];

						foreach ( $woofs_keys as $woofs_key ) {
							unset( $cart->cart_contents[ $woofs_key ] );
						}
					}
				}

				function woofs_new_order_item( $item_id, $item, $order_id ) {
					$order = wc_get_order( $order_id );

					$quantity = $item->get_quantity();

					if ( ( 'line_item' === $item->get_type() ) && ! wc_get_order_item_meta( $item_id, '_woofs_ids', true ) && ! wc_get_order_item_meta( $item_id, '_woofs_parent_id', true ) ) {
						$product    = $item->get_product();
						$product_id = $product->get_id();

						if ( $product && ( $woofs_ids = get_post_meta( $product_id, 'woofs_ids', true ) ) ) {
							if ( $woofs_items = $this->woofs_get_items( $woofs_ids, $product_id ) ) {
								foreach ( $woofs_items as $woofs_item ) {
									$woofs_item_id      = $woofs_item['id'];
									$woofs_item_price   = $woofs_item['price'];
									$woofs_item_qty     = $woofs_item['qty'];
									$woofs_item_product = wc_get_product( $woofs_item_id );

									if ( ! $woofs_item_product ) {
										continue;
									}

									if ( get_post_meta( $product_id, 'woofs_separately', true ) !== 'on' ) {
										$woofs_product_price  = $woofs_item_product->get_price();
										$woofs_item_new_price = $this->woofs_new_price( $woofs_product_price, $woofs_item_price );
										$woofs_item_product->set_price( $woofs_item_new_price );
									}

									// add force sells products
									$woofs_order_item_id = $order->add_product( $woofs_item_product, $woofs_item_qty * $quantity );

									if ( ! $woofs_order_item_id ) {
										continue;
									}

									$woofs_order_items = $order->get_items( 'line_item' );
									$woofs_order_item  = $woofs_order_items[ $woofs_order_item_id ];

									if ( get_post_meta( $product_id, 'woofs_separately', true ) !== 'on' ) {
										$woofs_order_item->add_meta_data( '_woofs_parent_id', $product_id, true );
									}

									$woofs_order_item->save();
								}
							}
						}

						$order->save();
					}
				}

				function woofs_cart_item_remove_link( $link, $cart_item_key ) {
					if ( isset( WC()->cart->cart_contents[ $cart_item_key ]['woofs_parent_key'] ) ) {
						$woofs_parent_key = WC()->cart->cart_contents[ $cart_item_key ]['woofs_parent_key'];

						if ( isset( WC()->cart->cart_contents[ $woofs_parent_key ] ) ) {
							return '';
						}
					}

					return $link;
				}

				function woofs_cart_contents_count( $count ) {
					// count for cart contents
					$cart_contents_count = get_option( '_woofs_cart_contents_count', 'both' );

					if ( $cart_contents_count !== 'both' ) {
						$cart_contents = WC()->cart->cart_contents;

						foreach ( $cart_contents as $cart_item_key => $cart_item ) {
							if ( ( $cart_contents_count === 'linked' ) && ! empty( $cart_item['woofs_ids'] ) ) {
								$count -= $cart_item['quantity'];
							}

							if ( ( $cart_contents_count === 'main' ) && ! empty( $cart_item['woofs_parent_id'] ) ) {
								$count -= $cart_item['quantity'];
							}
						}
					}

					return $count;
				}

				function woofs_get_item_count( $count, $item_type, $order ) {
					// count for order items
					$cart_contents_count = get_option( '_woofs_cart_contents_count', 'both' );
					$order_main          = $order_linked = 0;

					if ( $cart_contents_count !== 'both' ) {
						$order_items = $order->get_items( 'line_item' );

						foreach ( $order_items as $order_item ) {
							if ( $order_item->get_meta( '_woofs_parent_id' ) ) {
								$order_linked += $order_item->get_quantity();
							}

							if ( $order_item->get_meta( '_woofs_ids' ) ) {
								$order_main += $order_item->get_quantity();
							}
						}

						if ( ( $cart_contents_count === 'linked' ) && ( $order_linked > 0 ) ) {
							return $count - $order_main;
						}

						if ( ( $cart_contents_count === 'main' ) && ( $order_main > 0 ) ) {
							return $count - $order_linked;
						}
					}

					return $count;
				}

				function woofs_item_name( $name, $item ) {
					if ( isset( $item['woofs_parent_id'] ) && ! empty( $item['woofs_parent_id'] ) ) {
						$linked_text = self::woofs_localization( 'linked', esc_html__( '(linked to %s)', 'wpc-force-sells' ) );

						if ( strpos( $name, '</a>' ) !== false ) {
							$woofs_name = sprintf( $linked_text, '<a href="' . get_permalink( $item['woofs_parent_id'] ) . '">' . get_the_title( $item['woofs_parent_id'] ) . '</a>' );
						} else {
							$woofs_name = sprintf( $linked_text, get_the_title( $item['woofs_parent_id'] ) );
						}

						$name .= ' ' . apply_filters( 'woofs_item_associated', $woofs_name, $item );
						$name = apply_filters( 'woofs_item_name', $name, $item );
					}

					return $name;
				}

				function woofs_cart_item_quantity( $quantity, $cart_item_key, $cart_item ) {
					// show qty as text - not input
					if ( isset( $cart_item['woofs_parent_id'] ) ) {
						return $cart_item['quantity'];
					}

					return $quantity;
				}

				function woofs_add_to_cart_validation( $passed, $product_id ) {
					if ( ( get_post_meta( $product_id, 'woofs_separately', true ) !== 'on' ) && ( $woofs_ids = get_post_meta( $product_id, 'woofs_ids', true ) ) ) {
						if ( isset( $_POST['woofs_ids'] ) ) {
							$woofs_ids = $this->woofs_clean_ids( $_POST['woofs_ids'] );
						}

						if ( $woofs_items = $this->woofs_get_items( $woofs_ids, $product_id ) ) {
							foreach ( $woofs_items as $woofs_item ) {
								$woofs_product = wc_get_product( $woofs_item['id'] );

								if ( ! $woofs_product ) {
									wc_add_notice( esc_html__( 'One of the linked products is unavailable.', 'wpc-force-sells' ), 'error' );
									wc_add_notice( esc_html__( 'You cannot add this product to the cart.', 'wpc-force-sells' ), 'error' );

									return false;
								}

								if ( $woofs_product->is_type( 'variable' ) || ! $woofs_product->is_purchasable() || ! $woofs_product->is_in_stock() || ! $woofs_product->has_enough_stock( $woofs_item['qty'] ) ) {
									wc_add_notice( sprintf( esc_html__( '"%s" is un-purchasable.', 'wpc-force-sells' ), esc_html( $woofs_product->get_name() ) ), 'error' );
									wc_add_notice( esc_html__( 'You cannot add this product to the cart.', 'wpc-force-sells' ), 'error' );

									return false;
								}
							}
						}
					}

					return $passed;
				}

				function woofs_add_to_cart( $cart_item_key, $product_id, $quantity, $variation_id, $variation, $cart_item_data ) {
					if ( isset( $cart_item_data['woofs_ids'] ) && ! empty( $cart_item_data['woofs_ids'] ) ) {
						if ( $woofs_items = $this->woofs_get_items( $cart_item_data['woofs_ids'], $product_id ) ) {
							// add child products
							foreach ( $woofs_items as $woofs_item ) {
								$woofs_item_id           = $woofs_item['id'];
								$woofs_item_price        = $woofs_item['price'];
								$woofs_item_qty          = $woofs_item['qty'];
								$woofs_item_variation    = $woofs_item['attrs'];
								$woofs_item_variation_id = 0;
								$woofs_item_product      = wc_get_product( $woofs_item_id );

								if ( ! $woofs_item_product ) {
									continue;
								}

								$woofs_item_product_price = $woofs_item_product->get_price();

								if ( $woofs_item_product instanceof WC_Product_Variation ) {
									// ensure we don't add a variation to the cart directly by variation ID
									$woofs_item_variation_id = $woofs_item_id;
									$woofs_item_id           = $woofs_item_product->get_parent_id();

									if ( empty( $woofs_item_variation ) ) {
										$woofs_item_variation = $woofs_item_product->get_variation_attributes();
									}
								}

								if ( $woofs_item_product && $woofs_item_product->is_in_stock() && $woofs_item_product->is_purchasable() && $woofs_item_product->has_enough_stock( $woofs_item_qty ) ) {
									if ( get_post_meta( $product_id, 'woofs_separately', true ) !== 'on' ) {
										// calc new price
										$woofs_item_new_price = $this->woofs_new_price( $woofs_item_product_price, $woofs_item_price );

										// set price for child product
										$woofs_item_product->set_price( $woofs_item_new_price );

										$woofs_item_data = array(
											'woofs_parent_id'  => $product_id,
											'woofs_parent_key' => $cart_item_key,
											'woofs_price'      => $woofs_item_new_price,
											'woofs_qty'        => $woofs_item_qty
										);
										$woofs_cart_id   = WC()->cart->generate_cart_id( $woofs_item_id, $woofs_item_variation_id, $woofs_item_variation, $woofs_item_data );
										$woofs_item_key  = WC()->cart->find_product_in_cart( $woofs_cart_id );

										// add to cart
										if ( empty( $woofs_item_key ) ) {
											$woofs_item_key = WC()->cart->add_to_cart( $woofs_item_id, $woofs_item_qty * $quantity, $woofs_item_variation_id, $woofs_item_variation, $woofs_item_data );

											if ( $woofs_item_key ) {
												WC()->cart->cart_contents[ $cart_item_key ]['woofs_keys'][] = $woofs_item_key;
											}
										}
									} else {
										WC()->cart->add_to_cart( $woofs_item_id, $woofs_item_qty * $quantity, $woofs_item_variation_id, $woofs_item_variation );
									}
								}
							}
						}
					}
				}

				function woofs_add_cart_item_data( $cart_item_data, $product_id ) {
					if ( ! isset( $cart_item_data['woofs_parent_id'] ) && ( $woofs_ids = get_post_meta( $product_id, 'woofs_ids', true ) ) ) {
						if ( isset( $_POST['woofs_ids'] ) ) {
							$woofs_ids = $this->woofs_clean_ids( $_POST['woofs_ids'] );
							unset( $_POST['woofs_ids'] );
						}

						if ( ! empty( $woofs_ids ) ) {
							$cart_item_data['woofs_ids']        = $woofs_ids;
							$cart_item_data['woofs_separately'] = get_post_meta( $product_id, 'woofs_separately', true ) ?: 'off';
						}
					}

					return $cart_item_data;
				}

				function woofs_get_cart_item_from_session( $cart_item, $item_session_values ) {
					if ( isset( $item_session_values['woofs_ids'] ) && ! empty( $item_session_values['woofs_ids'] ) ) {
						$cart_item['woofs_ids'] = $item_session_values['woofs_ids'];
					}

					if ( isset( $item_session_values['woofs_parent_id'] ) ) {
						$cart_item['woofs_parent_id']  = $item_session_values['woofs_parent_id'];
						$cart_item['woofs_parent_key'] = $item_session_values['woofs_parent_key'];
						$cart_item['woofs_price']      = $item_session_values['woofs_price'];
						$cart_item['woofs_qty']        = $item_session_values['woofs_qty'];
					}

					return $cart_item;
				}

				function woofs_add_order_item_meta( $item, $cart_item_key, $values ) {
					// add _ to hide
					if ( isset( $values['woofs_parent_id'] ) ) {
						$item->update_meta_data( '_woofs_parent_id', $values['woofs_parent_id'] );
					}

					if ( isset( $values['woofs_ids'] ) ) {
						$item->update_meta_data( '_woofs_ids', $values['woofs_ids'] );
					}
				}

				function woofs_hidden_order_item_meta( $hidden ) {
					return array_merge( $hidden, array(
						'_woofs_parent_id',
						'_woofs_ids',
						'woofs_parent_id',
						'woofs_ids'
					) );
				}

				function woofs_before_order_item_meta( $item_id ) {
					if ( $woofs_parent_id = wc_get_order_item_meta( $item_id, '_woofs_parent_id', true ) ) {
						$linked_text = self::woofs_localization( 'linked', esc_html__( '(linked to %s)', 'wpc-force-sells' ) );

						echo sprintf( $linked_text, get_the_title( $woofs_parent_id ) );
					}
				}

				function woofs_item_visible( $visible, $item ) {
					if ( isset( $item['woofs_parent_id'] ) ) {
						return false;
					}

					return $visible;
				}

				function woofs_display_post_states( $states, $post ) {
					if ( 'product' == get_post_type( $post->ID ) ) {
						if ( $woofs_ids = get_post_meta( $post->ID, 'woofs_ids', true ) ) {
							$woofs_ids = $this->woofs_clean_ids( $woofs_ids );

							if ( ! empty( $woofs_ids ) ) {
								$woofs_count = count( explode( ',', $woofs_ids ) );
								$states[]    = apply_filters( 'woofs_post_states', '<span class="woofs-state">' . sprintf( esc_html__( 'Force sells (%s)', 'wpc-force-sells' ), $woofs_count ) . '</span>', $woofs_count, $post->ID );
							}
						}
					}

					return $states;
				}

				function woofs_get_search_results() {
					$keyword     = esc_html( $_POST['woofs_keyword'] );
					$id          = absint( $_POST['woofs_id'] );
					$exclude_ids = array( $id );
					$added_ids   = explode( ',', $this->woofs_clean_ids( $_POST['woofs_ids'] ) );

					if ( ( get_option( '_woofs_search_id', 'no' ) === 'yes' ) && is_numeric( $keyword ) ) {
						// search by id
						$woofs_query_args = array(
							'p'         => absint( $keyword ),
							'post_type' => 'product'
						);
					} else {
						$woofs_query_args = array(
							'is_woofs'       => true,
							'post_type'      => 'product',
							'post_status'    => 'publish',
							's'              => $keyword,
							'posts_per_page' => get_option( '_woofs_search_limit', 10 )
						);

						if ( get_option( '_woofs_search_same', 'no' ) !== 'yes' ) {
							if ( is_array( $added_ids ) && count( $added_ids ) > 0 ) {
								foreach ( $added_ids as $added_id ) {
									$added_id_new  = explode( '/', $added_id );
									$exclude_ids[] = absint( $added_id_new[0] ?: 0 );
								}
							}

							$woofs_query_args['post__not_in'] = $exclude_ids;
						}
					}

					$woofs_query = new WP_Query( $woofs_query_args );

					if ( $woofs_query->have_posts() ) {
						echo '<ul>';

						while ( $woofs_query->have_posts() ) {
							$woofs_query->the_post();
							$product = wc_get_product( get_the_ID() );

							if ( ! $product ) {
								continue;
							}

							$this->woofs_product_data_li( $product, '100%', 1, true );

							if ( $product->is_type( 'variable' ) ) {
								// show all childs
								$childs = $product->get_children();

								if ( is_array( $childs ) && count( $childs ) > 0 ) {
									foreach ( $childs as $child ) {
										$product_child = wc_get_product( $child );

										if ( $product_child ) {
											$this->woofs_product_data_li( $product_child, '100%', 1, true );
										}
									}
								}
							}
						}

						echo '</ul>';
						wp_reset_postdata();
					} else {
						echo '<ul><span>' . sprintf( esc_html__( 'No results found for "%s"', 'wpc-force-sells' ), $keyword ) . '</span></ul>';
					}

					die();
				}

				function woofs_product_data_li( $product, $price = '100%', $qty = 1, $search = false ) {
					$product_id    = $product->get_id();
					$product_class = ! $product->is_in_stock() ? 'out-of-stock' : '';

					if ( ! in_array( $product->get_type(), self::$woofs_types, true ) ) {
						$product_class .= ' disabled';
					}

					if ( $search ) {
						$remove_btn = '<span class="remove hint--left" aria-label="' . esc_html__( 'Add', 'wpc-force-sells' ) . '">+</span>';
					} else {
						$remove_btn = '<span class="remove hint--left" aria-label="' . esc_html__( 'Remove', 'wpc-force-sells' ) . '">×</span>';
					}

					if ( class_exists( 'WPCleverWoopq' ) && ( get_option( '_woopq_decimal', 'no' ) === 'yes' ) ) {
						$step = '0.000001';
					} else {
						$step = '1';
						$qty  = (int) $qty;
					}

					echo '<li class="' . esc_attr( trim( $product_class ) ) . '" data-id="' . $product->get_id() . '"><span class="move"></span><span class="price hint--right" aria-label="' . esc_html__( 'New price in percent or number', 'wpc-force-sells' ) . '"><input type="text" value="' . $price . '"/></span><span class="qty hint--right" aria-label="' . esc_html__( 'Quantity', 'wpc-force-sells' ) . '"><input type="number" value="' . $qty . '" min="' . $step . '" step="' . $step . '"/></span> <span class="data">' . strip_tags( $product->get_name() ) . ' (' . $product->get_price_html() . ')</span> <span class="type"><a href="' . get_edit_post_link( $product_id ) . '" target="_blank">' . $product->get_type() . '<br/>#' . $product->get_id() . '</a></span> ' . $remove_btn . '</li>';
				}

				function woofs_product_data_tabs( $tabs ) {
					$tabs['woofs'] = array(
						'label'  => esc_html__( 'Force Sells', 'wpc-force-sells' ),
						'target' => 'woofs_settings',
					);

					return $tabs;
				}

				function woofs_product_data_panels() {
					global $post;
					$post_id = $post->ID;
					?>
                    <div id='woofs_settings' class='panel woocommerce_options_panel woofs_table'>
                        <table>
                            <tr>
                                <th><?php esc_html_e( 'Search', 'wpc-force-sells' ); ?> (<a
                                            href="<?php echo admin_url( 'admin.php?page=wpclever-woofs&tab=settings#search' ); ?>"
                                            target="_blank"><?php esc_html_e( 'settings', 'wpc-force-sells' ); ?></a>)
                                </th>
                                <td>
                                    <div class="w100">
								<span class="loading"
                                      id="woofs_loading"
                                      style="display:none;"><?php esc_html_e( 'searching...', 'wpc-force-sells' ); ?></span>
                                        <input type="search" id="woofs_keyword"
                                               placeholder="<?php esc_html_e( 'Type any keyword to search', 'wpc-force-sells' ); ?>"/>
                                        <div id="woofs_results" class="woofs_results" style="display:none;"></div>
                                    </div>
                                </td>
                            </tr>
                            <tr class="woofs_tr_space">
                                <th><?php esc_html_e( 'Selected', 'wpc-force-sells' ); ?></th>
                                <td>
                                    <div class="w100">
                                        <input type="hidden" id="woofs_id" name="woofs_id"
                                               value="<?php echo esc_attr( $post_id ); ?>"/>
                                        <input type="hidden" id="woofs_ids" name="woofs_ids"
                                               value="<?php echo get_post_meta( $post_id, 'woofs_ids', true ); ?>"
                                               readonly/>
                                        <div id="woofs_selected" class="woofs_selected">
                                            <ul>
												<?php
												if ( $woofs_ids = get_post_meta( $post_id, 'woofs_ids', true ) ) {
													if ( $woofs_items = $this->woofs_get_items( $woofs_ids ) ) {
														foreach ( $woofs_items as $woofs_item ) {
															$woofs_item_id    = $woofs_item['id'];
															$woofs_item_price = $woofs_item['price'];
															$woofs_item_qty   = $woofs_item['qty'];
															$woofs_product    = wc_get_product( $woofs_item_id );

															if ( ! $woofs_product || ! in_array( $woofs_product->get_type(), self::$woofs_types, true ) ) {
																continue;
															}

															$this->woofs_product_data_li( $woofs_product, $woofs_item_price, $woofs_item_qty, false );
														}
													}
												}
												?>
                                            </ul>
                                        </div>
                                    </div>
                                </td>
                            </tr>
                            <tr class="woofs_tr_space">
                                <th><?php esc_html_e( 'Add separately', 'wpc-force-sells' ); ?></th>
                                <td>
                                    <input id="woofs_separately" name="woofs_separately"
                                           type="checkbox" <?php echo esc_attr( get_post_meta( $post_id, 'woofs_separately', true ) === 'on' ? 'checked' : '' ); ?>/>
                                    <span class="woocommerce-help-tip"
                                          data-tip="<?php esc_attr_e( 'If enabled, the force sell products will be added as separate items and stay unaffected from the main product, their prices will change back to the original.', 'wpc-force-sells' ); ?>"></span>
                                </td>
                            </tr>
                            <tr class="woofs_tr_space">
                                <th><?php esc_html_e( 'Above text', 'wpc-force-sells' ); ?></th>
                                <td>
                                    <div class="w100">
                                        <input type="text" name="woofs_before_text"
                                               value='<?php echo stripslashes( get_post_meta( $post_id, 'woofs_before_text', true ) ); ?>'/>
                                    </div>
                                </td>
                            </tr>
                            <tr class="woofs_tr_space">
                                <th><?php esc_html_e( 'Under text', 'wpc-force-sells' ); ?></th>
                                <td>
                                    <div class="w100">
                                        <input type="text" name="woofs_after_text"
                                               value='<?php echo stripslashes( get_post_meta( $post_id, 'woofs_after_text', true ) ); ?>'/>
                                    </div>
                                </td>
                            </tr>
                        </table>
                    </div>
					<?php
				}

				function woofs_save_option_field( $post_id ) {
					if ( isset( $_POST['woofs_ids'] ) ) {
						$woofs_ids = $this->woofs_clean_ids( sanitize_text_field( $_POST['woofs_ids'] ) );
						update_post_meta( $post_id, 'woofs_ids', $woofs_ids );
					}

					if ( isset( $_POST['woofs_before_text'] ) && ( $_POST['woofs_before_text'] !== '' ) ) {
						update_post_meta( $post_id, 'woofs_before_text', sanitize_textarea_field( $_POST['woofs_before_text'] ) );
					} else {
						delete_post_meta( $post_id, 'woofs_before_text' );
					}

					if ( isset( $_POST['woofs_after_text'] ) && ( $_POST['woofs_after_text'] !== '' ) ) {
						update_post_meta( $post_id, 'woofs_after_text', sanitize_textarea_field( $_POST['woofs_after_text'] ) );
					} else {
						delete_post_meta( $post_id, 'woofs_after_text' );
					}

					if ( isset( $_POST['woofs_separately'] ) ) {
						update_post_meta( $post_id, 'woofs_separately', 'on' );
					} else {
						update_post_meta( $post_id, 'woofs_separately', 'off' );
					}
				}

				function woofs_add_to_cart_form() {
					global $product;

					if ( ! $product->is_type( 'grouped' ) && get_post_meta( $product->get_id(), 'woofs_ids', true ) ) {
						wp_enqueue_script( 'wc-add-to-cart-variation' );
						$this->woofs_show_items();
					}
				}

				function woofs_add_to_cart_button() {
					global $product;

					if ( ! $product->is_type( 'grouped' ) && get_post_meta( $product->get_id(), 'woofs_ids', true ) ) {
						echo '<input name="woofs_ids" class="woofs_ids woofs-ids" type="hidden" value="' . get_post_meta( $product->get_id(), 'woofs_ids', true ) . '"/>';
					}
				}

				function woofs_has_variables( $items ) {
					foreach ( $items as $item ) {
						$item_id       = $item['id'];
						$woofs_product = wc_get_product( $item_id );

						if ( $woofs_product && $woofs_product->is_type( 'variable' ) ) {
							return true;
						}
					}

					return false;
				}

				function woofs_show_items() {
					global $product;
					$product_id = $product->get_id();
					$order      = 1;

					if ( $woofs_ids = get_post_meta( $product_id, 'woofs_ids', true ) ) {
						if ( $woofs_items = $this->woofs_get_items( $woofs_ids, $product_id ) ) {
							echo '<div class="woofs-wrap">';

							if ( $woofs_before_text = apply_filters( 'woofs_before_text', get_post_meta( $product_id, 'woofs_before_text', true ) ?: self::woofs_localization( 'above_text' ), $product_id ) ) {
								echo '<div class="woofs-before-text woofs-text">' . esc_html( $woofs_before_text ) . '</div>';
							}
							?>
                            <table class="woofs-products woofs-table"
                                   data-product-type="<?php echo esc_attr( $product->get_type() ); ?>"
                                   data-variables="<?php echo esc_attr( $this->woofs_has_variables( $woofs_items ) ? 'yes' : 'no' ); ?>"
                                   data-product-sku="<?php echo esc_attr( $product->get_sku() ); ?>"
                                   data-product-price="<?php echo esc_attr( $product->get_type() === 'variable' ? '0' : wc_get_price_to_display( $product ) ); ?>">
                                <tbody>
								<?php foreach ( $woofs_items as $woofs_item ) {
									$woofs_item_id    = $woofs_item['id'];
									$woofs_item_price = $woofs_item['price'];
									$woofs_item_qty   = $woofs_item['qty'];

									if ( ! class_exists( 'WPCleverWoopq' ) || ( get_option( '_woopq_decimal', 'no' ) !== 'yes' ) ) {
										$woofs_item_qty = (int) $woofs_item_qty;
									}

									$woofs_product = wc_get_product( $woofs_item_id );

									if ( ! $woofs_product || ! in_array( $woofs_product->get_type(), self::$woofs_types, true ) ) {
										continue;
									}
									?>
                                    <tr class="woofs-product"
                                        data-name="<?php echo esc_attr( $woofs_product->get_name() ); ?>"
                                        data-id="<?php echo esc_attr( $woofs_product->is_type( 'variable' ) || ! $woofs_product->is_in_stock() ? 0 : $woofs_item_id ); ?>"
                                        data-price="<?php echo esc_attr( $woofs_item_price ); ?>"
                                        data-price-ori="<?php echo esc_attr( wc_get_price_to_display( $woofs_product ) ); ?>"
                                        data-qty="<?php echo esc_attr( $woofs_item_qty ); ?>"
                                        data-order="<?php echo esc_attr( $order ); ?>">
										<?php if ( get_option( '_woofs_show_thumb', 'yes' ) !== 'no' ) { ?>
                                            <td class="woofs-thumb">
                                                <div class="woofs-thumb-ori">
													<?php echo $woofs_product->get_image(); ?>
                                                </div>
                                                <div class="woofs-thumb-new"></div>
                                            </td>
										<?php } ?>
                                        <td class="woofs-title">
											<?php
											// quantity
											echo apply_filters( 'woofs_item_qty', $woofs_item_qty . ' × ', $woofs_item_qty, $woofs_product );

											// product name
											if ( $woofs_product->is_in_stock() ) {
												$woofs_product_name = $woofs_product->get_name();
											} else {
												$woofs_product_name = '<s>' . $woofs_product->get_name() . '</s>';
											}

											if ( get_option( '_woofs_link', 'yes' ) !== 'no' ) {
												echo '<a ' . ( get_option( '_woofs_link', 'yes' ) === 'yes_popup' ? 'class="woosq-link" data-id="' . $woofs_item_id . '"' : '' ) . ' href="' . get_permalink( $woofs_item_id ) . '" ' . ( get_option( '_woofs_link', 'yes' ) === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . $woofs_product_name . '</a>';
											} else {
												echo $woofs_product_name;
											}

											if ( get_option( '_woofs_show_description', 'no' ) === 'yes' ) {
												echo '<div class="woofs-description">' . $woofs_product->get_short_description() . '</div>';
											}

											echo '<div class="woofs-availability">' . wc_get_stock_html( $woofs_product ) . '</div>';
											?>
                                        </td>
										<?php if ( get_option( '_woofs_show_price', 'yes' ) !== 'no' ) { ?>
                                            <td class="woofs-price">
                                                <div class="woofs-price-new"></div>
                                                <div class="woofs-price-ori">
													<?php if ( ( $woofs_item_price !== '100%' ) && ( get_post_meta( $product_id, 'woofs_separately', true ) !== 'on' ) ) {
														echo '<del>' . wc_price( $woofs_product->get_price() ) . '</del> ' . wc_price( $this->woofs_new_price( $woofs_product->get_price(), $woofs_item_price ) );
													} else {
														echo $woofs_product->get_price_html();
													} ?>
                                                </div>
                                            </td>
										<?php } ?>
                                    </tr>
									<?php
									$order ++;
								} ?>
                                </tbody>
                            </table>
							<?php
							echo '<div class="woofs-total woofs-text"></div>';

							echo '<div class="woofs-alert woofs-text" style="display: none"></div>';

							if ( $woofs_after_text = apply_filters( 'woofs_after_text', get_post_meta( $product_id, 'woofs_after_text', true ) ?: self::woofs_localization( 'under_text' ), $product_id ) ) {
								echo '<div class="woofs-after-text woofs-text">' . esc_html( $woofs_after_text ) . '</div>';
							}

							echo '</div>';
						}
					}
				}

				function woofs_get_cart_contents( $cart_contents ) {
					foreach ( $cart_contents as $cart_item_key => $cart_item ) {
						if ( isset( $cart_item['woofs_parent_id'], $cart_item['woofs_price'] ) ) {
							// remove orphaned products
							if ( isset( $cart_item['woofs_parent_key'] ) && ! empty( $cart_item['woofs_parent_key'] ) ) {
								$parent_key = $cart_item['woofs_parent_key'];

								if ( ! isset( $cart_contents[ $parent_key ] ) && ! ( $cart_item['data'] instanceof WC_Product_Subscription ) && ! ( $cart_item['data'] instanceof WC_Product_Subscription_Variation ) ) {
									unset( $cart_contents[ $cart_item_key ] );
									continue;
								}
							}

							$product_id = $cart_item['product_id'];

							if ( ! $product_id ) {
								continue;
							}

							$new_price = $this->woofs_new_price( $cart_item['data']->get_price(), $cart_item['woofs_price'] );
							$cart_item['data']->set_price( $new_price );

							// sync quantity
							if ( ! empty( $cart_item['woofs_parent_key'] ) && ! empty( $cart_item['woofs_qty'] ) ) {
								$parent_key = $cart_item['woofs_parent_key'];

								if ( isset( $cart_contents[ $parent_key ] ) ) {
									$cart_contents[ $cart_item_key ]['quantity'] = $cart_item['woofs_qty'] * $cart_contents[ $parent_key ]['quantity'];
								} elseif ( ( $parent_new_key = array_search( $parent_key, array_column( $cart_contents, 'woofs_key', 'key' ) ) ) !== false ) {
									$cart_contents[ $cart_item_key ]['quantity'] = $cart_item['woofs_qty'] * $cart_contents[ $parent_new_key ]['quantity'];
								}
							}
						}
					}

					return $cart_contents;
				}

				function woofs_search_sku( $query ) {
					if ( $query->is_search && isset( $query->query['is_woofs'] ) ) {
						global $wpdb;
						$sku = $query->query['s'];
						$ids = $wpdb->get_col( $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key='_sku' AND meta_value = %s;", $sku ) );

						if ( ! $ids ) {
							return;
						}

						unset( $query->query['s'], $query->query_vars['s'] );
						$query->query['post__in'] = array();

						foreach ( $ids as $id ) {
							$post = get_post( $id );

							if ( $post->post_type === 'product_variation' ) {
								$query->query['post__in'][]      = $post->post_parent;
								$query->query_vars['post__in'][] = $post->post_parent;
							} else {
								$query->query_vars['post__in'][] = $post->ID;
							}
						}
					}
				}

				function woofs_search_exact( $query ) {
					if ( $query->is_search && isset( $query->query['is_woofs'] ) ) {
						$query->set( 'exact', true );
					}
				}

				function woofs_search_sentence( $query ) {
					if ( $query->is_search && isset( $query->query['is_woofs'] ) ) {
						$query->set( 'sentence', true );
					}
				}

				function woofs_get_items( $ids, $product_id = null ) {
					$woofs_items = array();
					$ids         = $this->woofs_clean_ids( $ids, $product_id );

					if ( ! empty( $ids ) ) {
						$items = explode( ',', $ids );

						if ( is_array( $items ) && count( $items ) > 0 ) {
							foreach ( $items as $item ) {
								$item_arr      = explode( '/', $item );
								$woofs_items[] = array(
									'id'    => apply_filters( 'woofs_item_id', absint( isset( $item_arr[0] ) ? $item_arr[0] : 0 ) ),
									'price' => isset( $item_arr[1] ) ? $this->woofs_format_price( $item_arr[1] ) : '100%',
									'qty'   => (float) ( isset( $item_arr[2] ) ? $item_arr[2] : 1 ),
									'attrs' => isset( $item_arr[3] ) ? (array) json_decode( rawurldecode( $item_arr[3] ) ) : array()
								);
							}
						}
					}

					if ( count( $woofs_items ) > 0 ) {
						return apply_filters( 'woofs_get_items', $woofs_items, $product_id );
					}

					return apply_filters( 'woofs_get_items', false, $product_id );
				}

				function woofs_clean_ids( $ids, $product_id = null ) {
					return apply_filters( 'woofs_clean_ids', $ids, $product_id );
				}

				function woofs_format_price( $price ) {
					// format price to percent or number
					$price = preg_replace( '/[^.%0-9]/', '', $price );

					return apply_filters( 'woofs_format_price', $price );
				}

				function woofs_new_price( $old_price, $new_price ) {
					if ( strpos( $new_price, '%' ) !== false ) {
						$calc_price = ( (int) $new_price * $old_price ) / 100;
					} else {
						$calc_price = $new_price;
					}

					return $calc_price;
				}
			}

			new WPClever_Woofs();
		}
	}
} else {
	add_action( 'admin_notices', 'woofs_notice_premium' );
}

if ( ! function_exists( 'woofs_notice_wc' ) ) {
	function woofs_notice_wc() {
		?>
        <div class="error">
            <p><strong>WPC Force Sells</strong> requires WooCommerce version 3.0 or greater.</p>
        </div>
		<?php
	}
}

if ( ! function_exists( 'woofs_notice_premium' ) ) {
	function woofs_notice_premium() {
		?>
        <div class="error">
            <p>Seems you're using both free and premium version of <strong>WPC Force Sells</strong>.
                Please deactivate the free version when using the premium version.</p>
        </div>
		<?php
	}
}